@@ -42,7 +42,7 @@ shopt > /dev/null 2>&1 && USING_BASH=1
 if [ -z "$NEED_POSIX" ] && [ -z "$TRIED_DASH" ] &&
 	command -v dash > /dev/null 2>&1 ; then
 
-    if [ "`basename $0`" = "$0" ] ; then
+    if [ "${0##*/}" = "$0" ] ; then
 	myself=`command -v $0` || myself=
     elif [ -x "$0" ] ; then
 	myself=$0
@@ -57,8 +57,8 @@ unset TRIED_DASH
 SWSUSP_D="/etc/hibernate"
 SCRIPTLET_PATH="$SWSUSP_D/scriptlets.d /usr/local/share/hibernate/scriptlets.d /usr/share/hibernate/scriptlets.d"
 DEFAULT_CONFIG_FILE="$SWSUSP_D/hibernate.conf"
-EXE=`basename $0`
-VERSION="1.93"
+EXE="${0##*/}"
+VERSION="1.97"
 
 # Add these to the $PATH just in case.
 PATH="$PATH:/sbin:/usr/sbin:/usr/local/sbin"
@@ -89,10 +89,12 @@ vcat() {
     v="$1"
     shift
     if [ "$v" -le $VERBOSITY ] ; then
-	cat $@
+	cat "$@"
     else
 	if [ "$v" -le $LOG_VERBOSITY -a "$LOGPIPE" != "cat" ] ; then
-	    cat $@ | $LOGPIPE > /dev/null
+	    cat "$@" | $LOGPIPE > /dev/null
+	else
+	    cat > /dev/null
 	fi
     fi
 }
@@ -116,6 +118,13 @@ AddResumeHook() {
 }
 RESUME_BITS=""
 
+# AddTerminationHandler <function name>: adds the given function to the chain of
+# functions to handle termination cleanups.
+AddTerminationHandler() {
+    TERMINATION_HANDLERS="$TERMINATIONS_HANDLERS $1"
+}
+TERMINATION_HANDLERS=""
+
 # AddConfigHandler <function name>: adds the given function to the chain of
 # functions to handle extra configuration options.
 AddConfigHandler() {
@@ -209,15 +218,17 @@ FindXServer() {
 	fi
     done
     if [ -z $xuser ] ; then
-        for xpid in `pidof kwrapper ksmserver kdeinit gnome-session fvwm fvwm2 pwm blackbox fluxbox X XFree86 Xorg` ; do
+        for xpid in `pidof kwrapper ksmserver kdeinit gnome-session fvwm fvwm2 pwm blackbox fluxbox WindowMaker X XFree86 Xorg` ; do
 	    # Ensure the process still exists, and we aren't hallucinating.
 	    [ -d "/proc/$xpid/" ] || continue
 
-	    xauth=`awk 'BEGIN{RS="\\000";FS="="}($1 == "XAUTHORITY"){print $2}' < /proc/$xpid/environ`
-	    xhome=`awk 'BEGIN{RS="\\000";FS="="}($1 == "HOME"){print $2}' < /proc/$xpid/environ`
+	    xauth="`get_env_var_of_process XAUTHORITY $xpid`"
+	    xhome="`get_env_var_of_process HOME $xpid`"
 	    xuser=`/bin/ls -ld /proc/$xpid/ | awk '{print $3}'`
 	    [ -z $xauth ] && [ -n $xhome ] && [ -f $xhome/.Xauthority ] && xauth=$xhome/.Xauthority
 
+	    [ -z $xauth ] && continue
+
 	    XAUTHORITY=$xauth su $xuser -c "$xhost" > /dev/null 2>&1 && break
 
 	   xauth=
@@ -324,6 +335,16 @@ BEGIN {
 
 }
 
+# PluginTermination <params>: queries all loaded scriptlets if they want to
+# handle the given event. Returns 0 if the option was handled, 1 otherwise.
+PluginTermination() {
+    local i
+    for i in $TERMINATION_HANDLERS ; do
+	$i $@ && return 0
+    done
+    return 1
+}
+
 # PluginConfigOption <params>: queries all loaded scriptlets if they want to
 # handle the given option. Returns 0 if the option was handled, 1 otherwise.
 PluginConfigOption() {
@@ -472,7 +493,7 @@ CheckImplicitAlternateConfig() {
     [ -n "$CONFIG_FILE" ] && return 0
 
     local self
-    self=`basename $0`
+    self="${0##*/}"
     case $self in
 	hibernate-*) 
 	    CONFIG_FILE="$SWSUSP_D/${self#hibernate-}.conf"
@@ -532,7 +553,7 @@ LoadScriptlets() {
 	    case "$scriptlet" in *~|*.bak) continue ;; esac
 
 	    # Don't source a scriptlet by name more than once.
-	    scriptlet_name="`basename $scriptlet`"
+	    scriptlet_name="${scriptlet##*/}"
 
 	    eval "prev_path=\"\${HAVE_SOURCED_SCRIPTLET_$scriptlet_name}\""
 
@@ -666,9 +687,11 @@ ProcessConfigOption() {
 
 # ReadConfigFile: reads in a configuration file from stdin and sets the
 # appropriate variables in the script. Returns 0 on success, exits on errors
+conffiles_read=
 ReadConfigFile() {
     local option params
     local file_name="$1"
+    
     if [ ! -f "${file_name}" ] ; then
 	# Search in /etc/hibernate
 	if [ -f "$SWSUSP_D/$file_name" ] ; then
@@ -679,6 +702,12 @@ ReadConfigFile() {
 	    return 1
 	fi
     fi
+
+    if [ "${conffiles_read#*$file_name}" != "$conffiles_read" ]; then
+  	vecho 2 "$EXE: Skipping already included config file $file_name"
+  	return 0
+    fi
+
     while true ; do
 	# Doing the read this way allows means we don't require a new-line
 	# at the end of the file.
@@ -789,6 +818,14 @@ ctrlc_handler() {
     SUSPEND_ABORT=1
 }
 
+# Gets the value of the environment variable in the environment space of the
+# processes identified by its PID.
+get_env_var_of_process()
+{
+    local pid="$1" envvar="$2"
+    tr '\0' '\n' </proc/$pid/environ | sed -ne "s/^$envvar=//p"
+}
+
 ############################### MAIN #########################################
 
 # Some starting values:
@@ -861,8 +898,10 @@ else
     fi
 fi
 
+PluginTermination
+
 echo "Resumed at "`date` | $LOGPIPE > /dev/null
 
 exit $EXIT_CODE
 
-# $Id: hibernate.sh 1052 2006-07-12 10:15:00Z bernard $
+# $Id: hibernate.sh 1155 2007-09-30 13:54:58Z madduck $
